@@ -147,6 +147,11 @@ class Chart:
             '''
             Contour plot
             '''
+            if self.Funct.type == 'gaussian' and self.output_scalar_range[1] == [-1,1]:
+                levels = [0.1,0.3,0.5,0.7,0.9]
+            else:
+                levels = None
+
             x_mesh, y_mesh = np.meshgrid(np.linspace(self.input_scalar_range0[0][0],self.input_scalar_range0[0][1],50),
                                          np.linspace(self.input_scalar_range1[0][0],self.input_scalar_range1[0][1],50))
             CS = ax.contour(
@@ -156,12 +161,13 @@ class Chart:
                         Chart.minmaxscalar(x_mesh,scalar_ranges=self.input_scalar_range0),
                         Chart.minmaxscalar(y_mesh,scalar_ranges=self.input_scalar_range1)
                         ),
-                        scalar_ranges=self.output_scalar_range)
+                        scalar_ranges=self.output_scalar_range),
+                        levels = levels
                         )
             ax.clabel(CS, inline=1, fontsize=10) #label for contour plot
             return fig, ax
     
-    def ev(self,inputs,colour,fig,gs,loc):    
+    def ev(self,inputs,colour,fig,gs,loc,alpha):    
         '''
         Once we have the axis with the plot of the function, we now scatter the plot with the inputs. This is called evaluating the chart.
         Colour is a collection of numbers with the same size as an input.
@@ -179,28 +185,36 @@ class Chart:
                         Chart.minmaxscalar(inputs[0],scalar_ranges=self.input_scalar_range0)
                         ),
                         scalar_ranges=self.output_scalar_range),
-                        marker='o',c=colour
+                        alpha = alpha, marker='o',c=colour
             )
         else:
             '''
             Two dimensional plot. Observe that there isn't any evalation of the Function here.
             '''
-            cax = ax.scatter(x=inputs[0],y=inputs[1],c=colour)
+            cax = ax.scatter(x=inputs[0],y=inputs[1],alpha=alpha,c=colour)
             
         fig.colorbar(cax) #Adds the colour bar to the axes
         return fig,ax,cax
 
-class GraphChart:
-    def __init__(self,graph):#,data,colour=None,figsize=(20,10)):
-        self.graph = graph
-        #self.data = data
-        #if colour is None:
-        #    self.colour =  data[output_reg.name]  #colour of the plots
-        #else:
-        #    self.colour = colour
-        #self.fig = plt.figure(figsize=figsize)
+class GraphPlot:
+    """
+    A package to plot every interaction of a feyn._graph.Graph. To obtain the plot one first initalises the object with the graph.
+    Then one uses graph_eval to evaluate every interaction at each datapoint. Then one uses plot to plot the figure.
 
-    def model_ev(self,data):
+    Keyword arguments:
+    graph -- A feyn._graph.Graph object
+    """
+    def __init__(self,graph):
+        self.graph = graph
+        
+    def graph_eval(self,data):
+        """
+        Evaluates each interaction in the graph at every datapoint.
+
+        Keyword arguments:
+        data -- Can be either a dict mapping register names to value arrays, or a pandas.DataFrame. Evaluates each interaction of each datapoint.
+        Must include the target variable for default colour behaviour
+        """ 
         self.data = data
         interactions_evaluation = np.zeros((len(self.data),len(self.graph)))
         destandardiser = Funct(self.graph[-1]) #This is the destandardiser so that the output register activation is actually the destandardised value
@@ -211,9 +225,25 @@ class GraphChart:
             interactions_evaluation[i][-1] = destandardiser.ev(interactions_evaluation[i][-1]) #This is destandardising the last value in each row
         self.eval = interactions_evaluation
     
-    def plot(self,colour = None, figsize=(30,20)):
+    def plot(self,colour = None, figsize=(30,20),alpha=None):
+        """
+        Creates a figure that contains all plots of interactions
+
+        Keyword arguments:
+        colour -- colour of scatter plot. An array of numbers that is the same length of the data that the object has been evaluated on.
+        Default colour is the target variable of the data.
+
+        figsize -- a tuple that determines the size of the figure
+        alpha -- opacity of every scatter point. Takes values between 0 and 1.
+
+        Returns:
+        matplotlib.pyplot.figure -- A figure containing scatter plot of datapoints of all interactions in graph.
+        """
+        
         fig = plt.figure(figsize=figsize) #The figure that has all the subplots
-        coords, max_depth, max_height = GraphChart.chart_locations(self.graph)
+        plt.rc('axes', labelsize=14)    # fontsize of the x and y labels
+
+        coords, max_depth, max_height = GraphPlot._chart_locations(self.graph)
         gs = GridSpec(max_height,max_depth,figure=fig)
 
         output_reg = self.graph[-1] #The output register
@@ -256,8 +286,7 @@ class GraphChart:
                     input_scalar_ranges.append([[-1,1],[-1,1]]) #Range of input
                 
             chart.set_input_scalar_ranges(input_scalar_ranges)
-            #chart.set_input_range(input_ranges) #Setting input ranges
-            
+                        
             if location == (len(self.graph) - 2):
                 '''
                 If the interaction is the final one before the output register then we need to destandardise the output to the range of the target variable
@@ -265,14 +294,13 @@ class GraphChart:
                 feature_min = output_reg.state._to_dict()['feature_min']
                 feature_max = output_reg.state._to_dict()['feature_max']
                 chart.set_output_scalar_ranges([[-1,1],[feature_min,feature_max]])
-                #chart.set_output_scalar_range(output_range=(output_reg.state._to_dict()['feature_min'], output_reg.state._to_dict()['feature_max'])) #output range
             else:
                 '''
                 Otherwise the output range is [-1,1]
                 '''
                 chart.set_output_scalar_ranges()
 
-            fig, ax, cax = chart.ev(inputs,colour,fig,gs,coord) #this now plots and evaluates the chart from this interaction.
+            fig, ax, cax = chart.ev(inputs,colour,fig,gs,coord,alpha) #this now plots and evaluates the chart from this interaction.
             
             #Below is labelling the axes
             if len(inputs) == 1:
@@ -283,9 +311,10 @@ class GraphChart:
                 ax.set(xlabel=self.graph[interaction.sources[0]].name + ', loc: ' + str(self.graph[interaction.sources[0]]._location),
                     ylabel=self.graph[interaction.sources[1]].name + ', loc: ' + str(self.graph[interaction.sources[1]]._location),
                     title='Interaction ' + str(interaction._location) + ': ' + str(interaction.name))
+        return fig
 
     @staticmethod
-    def chart_locations(graph):
+    def _chart_locations(graph):
         '''
         Given a graph, this provides the coordinates needed for the gridreference in a plot
         '''
@@ -299,122 +328,3 @@ class GraphChart:
             '''
             precoord0 += [i for i in range(number)]        
         return list(zip(precoord0,precoord1)), max_depth, max_height
-
-
-#############################################################
-
-
-def chart_locations(graph):
-    '''
-    Given a graph, this provides the coordinates needed for the gridreference in a plot
-    '''
-    precoord1 = [graph[i].depth for i in range(len(graph)) if (graph[i].type != 'fixed' and graph[i].type !='cat')] #depth of each interaction that is not a register
-    max_height = max(Counter(precoord1).values()) #maximum amount of interactions in a row in the graph
-    max_depth = max(precoord1)+1 #maximum amount of interactions in a columns in the graph
-    precoord0 = []
-    for number in list(Counter(precoord1).values()): 
-        '''
-        To get the correct the coordinates the count has to restart to zero each time we move columns.
-        '''
-        precoord0 += [i for i in range(number)]        
-    return list(zip(precoord0,precoord1)), max_depth, max_height
-
-def _model_ev(graph,data):
-    '''
-    Given a graph with n interactions and the database with k rows, the returns a k x n numpy array where at the ith,jth point
-    is the value of datapoint i at interaction j. At the moment the activation at the output register is just a repeat of the last non-register interaction
-    '''
-    interactions_evaluation = np.zeros((len(data),len(graph)))
-    destandardiser = Funct(graph[-1]) #This is the destandardiser so that the output register activation is actually the destandardised value
-    for i in range(len(data)):
-        graph.predict(data[i:i+1]) #Activation only remembers the last point that has gone through the graph. So we need to predict first...
-        for j in range(len(graph)):
-            interactions_evaluation[i][j] = graph[j].activation #Then find the activation value
-        interactions_evaluation[i][-1] = destandardiser.ev(interactions_evaluation[i][-1]) #This is destandardising the last value in each row
-    return interactions_evaluation
-
-def model_chart(graph,data,colour=None,figsize=(20,10)):
-    '''
-    This produces a single figure with several subplots that are arranged somewhat like the graph.
-    A subplot is either from an interaction with a single input or from two inputs.
-    If its one input then the subplot has on the x-axis the input and the y-axis the output of the function in the interaction.
-    The line is the function and the scatter are the datapoints
-
-    If its two inputs then the subplot has on the x and y-axis the inputs and the output is represented as contour plots.
-    The scatter are the datapoints
-    '''
-    interactions = [graph[i] for i in range(len(graph)) if (graph[i].type != 'fixed' and graph[i].type !='cat')] #Every interaction the requires a chart
-    ie = _model_ev(graph,data) #every activation value for every datapoint
-    output_reg = graph[-1] #The output register
-
-    fig = plt.figure(figsize=figsize) #The figure that has all the subplots
-    coords, max_depth, max_height = chart_locations(graph) #coordinates for the gridspec, max_depth and max_height are the dimensions of the gridspec
-
-    gs = GridSpec(max_height,max_depth,figure=fig) #the gridspec helps organise the subplots so it's somewhat in the same format as the graph
-
-    
-    if colour is None:
-        colour =  data[output_reg.name]  #colour of the plots
-    
-    for interaction, coord in zip(interactions,coords):
-        location = interaction._location
-        chart = Chart(Funct(interaction))
-        inputs = []
-        input_scalar_ranges = []
-        for source in interaction.sources:
-            '''
-            This goes through each source of the interaction and determines the inputs 
-            '''
-            if graph[source].type == 'fixed':
-                '''
-                When the source is a numerical register then the inputs comes from the data.
-                '''
-                inputs.append(data[graph[source].name]) #Input
-                feature_min = graph[source].state._to_dict()['feature_min']
-                feature_max = graph[source].state._to_dict()['feature_max']
-                input_scalar_ranges.append([[feature_min, feature_max],[-1,1]]) #The ranges of the input for the chart
-            elif graph[source].type == 'cat':
-                '''
-                When the source is a categorical register then the inputs come from the weights at that register.
-                Fortunately this is done by the activation function at the register
-                '''
-                inputs.append([ie[i][source] for i in range(len(data))]) #Inputs
-                weight_min = min(graph[source].state._to_dict()['categories'], key=lambda x : x[1])[1] #minimum value
-                weight_max = max(graph[source].state._to_dict()['categories'], key=lambda x : x[1])[1] #maximum value
-                input_scalar_ranges.append([[weight_min,weight_max],[weight_min,weight_max]]) #Ranges of the input for the chart
-            else:
-                '''
-                When the source is from an interaction that is not a register then we retrieve the inputs from the activation function
-                '''
-                inputs.append([ie[i][source] for i in range(len(data))]) #Inputs
-                input_scalar_ranges.append([[-1,1],[-1,1]]) #Range of input
-        
-        chart.set_input_scalar_ranges(input_scalar_ranges)
-        #chart.set_input_range(input_ranges) #Setting input ranges
-        
-        if location == (len(graph) - 2):
-            '''
-            If the interaction is the final one before the output register then we need to destandardise the output to the range of the target variable
-            '''
-            feature_min = output_reg.state._to_dict()['feature_min']
-            feature_max = output_reg.state._to_dict()['feature_max']
-            chart.set_output_scalar_ranges([[-1,1],[feature_min,feature_max]])
-            #chart.set_output_scalar_range(output_range=(output_reg.state._to_dict()['feature_min'], output_reg.state._to_dict()['feature_max'])) #output range
-        else:
-            '''
-            Otherwise the output range is [-1,1]
-            '''
-            chart.set_output_scalar_ranges()
-
-        fig, ax, cax = chart.ev(inputs,colour,fig,gs,coord) #this now plots and evaluates the chart from this interaction.
-        
-        #Below is labelling the axes
-        if len(inputs) == 1:
-            ax.set(xlabel=graph[interaction.sources[0]].name + ', loc: ' + str(graph[interaction.sources[0]]._location),
-                   ylabel=interaction.name + ', loc: ' + str(interaction._location),
-                   title='Interaction ' + str(interaction._location)  + ': ' + str(interaction.name))
-        else:
-            ax.set(xlabel=graph[interaction.sources[0]].name + ', loc: ' + str(graph[interaction.sources[0]]._location),
-                   ylabel=graph[interaction.sources[1]].name + ', loc: ' + str(graph[interaction.sources[1]]._location),
-                   title='Interaction ' + str(interaction._location) + ': ' + str(interaction.name))
-    
